@@ -228,11 +228,15 @@ model.frame.Weka_classifier <-
 function(formula, ...)
 {
     if(!is.null(mf <- formula$model)) return(mf)
+    dots <- list(...)
+    nargs <- dots[match(c("data", "na.action", "subset"),
+                        names(dots), 0L)]
     mf <- formula$call
     mf <- mf[c(1L, match(c("formula", "data", "subset", "na.action"),
                          names(mf), 0L))]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
+    mf[names(nargs)] <- nargs
     if(is.null(env <- environment(formula$terms)))
         env <- parent.frame()
     .compose_and_funcall(formula$handlers$data, eval(mf, env))
